%macro graphit;
   /* roll up - summarizing predicted probabilities of each count */
   proc summary data=pred3 noprint;
      var pred_:;
      output out=pred4 sum=;
   run;

  /* swap horizontal to vertical - calculate percentages */
   data pred5(keep=&depvar. pred pct pctcum);
      set pred4(keep=pred_0-pred_&y_max. pred_sum);
      array preds(18) pred_0 pred_1 pred_2 pred_3 pred_4 pred_5 pred_6 pred_7 pred_8 
                      pred_9 pred_10 pred_11 pred_12 pred_13 pred_14 pred_15 pred_16 pred_17;
      array ns (18) (0 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17);
      do i = 1 to dim(preds);
         &depvar.=ns(i); 
         pred=preds(i);
         pct = (pred/pred_sum)*100;
         pctcum+pct;
         output;
      end;
   run;
   *proc print data=pred5;
    *  var &depvar. pred pct:;
    *  format pred 8. pct: comma10.2;
    *  sum pred;
   *run;

   *** actual counts ;
   proc freq data=&dataset. noprint;
      tables &depvar. / missing  out=pred6;
   run;

   /* bring together actuals vs. predicted & test */
   data pred7;
      merge pred6 pred5(keep=&depvar. pred pct);
      by &depvar.;
   run; 

   *proc print data=pred7;
   *   format pred 8.;
   *   sum _all_;
   *run;
   
   *** get the range for the y-axis;
proc sql noprint;
   select max(percent) into :max_pctc
      from pred7;
   select max(pct) into :max_pctp
      from pred7;
quit;

%let max_pctc=&max_pctc;
%let max_pctp=&max_pctp;
   
data _null_;
   max=max(&max_pctc., &max_pctp.);
   if mod(max, 10) < 5 then upper=round(max, 10)+ 10;
   else upper=round(max, 10);
   call symput('upper', upper);
run;

  /* generate a count histogram from the sample overlaid with the predicted distribution */
  /* PROC Gbarline is new in SAS9 */
goptions reset=all device=activex;
ods html style=journal;
axis1 order=0 to &upper. by 10; 
axis2 order=0 to &upper. by 10 label=none;
proc gbarline data=pred7;
   bar &depvar. / sumvar=percent type=mean discrete axis=axis1;
   plot / sumvar=pct raxis=axis2;
   title "&title";
run;
quit;
   ods html close;
   title;
%mend graphit;


%macro graphGP(title=, dataset=, depvar=);
   %macro estimate;
      %do i = 0 %to &y_max.;
         mu=pred;
         pred_&i.=(((1 + k*&i.)**(&i. - 1))/fact(&i.))*((mu/(1 + k*mu))**&i.)*
    exp(-(1 + k*&i.)*(mu/(1 + k*mu)));  	* GPD density;
      %end;
   %mend;

   data pred3;
      merge pred2 pred_k;
      array preds (*) pred_0-pred_&y_max.;
      %estimate;
      pred_sum = sum(of preds(*));	
   run;
   %graphit;
%mend;



%macro graphZIP(title=, dataset=, depvar=);
   %macro estimate;
      %do i = 0 %to &y_max.;
         if &i.=0 then
            pred_&i.=pred_zi + (1-pred_zi)*pdf('poisson',&i., pred); * add mass to zero counts;
         else pred_&i.=(1-pred_zi)*pdf('poisson',&i., pred);	* remove mass from gt zero counts;
      %end;
   %mend;

   data pred3;
      merge pred_zidata preddata;
      array preds (*) pred_0-pred_&y_max.;
      %estimate;
      pred_sum = sum(of preds(*));
   run;
   %graphit;
%mend;

%macro graphZINB(title=, dataset=, depvar=);
   %macro estimate;
      %do i = 0 %to &y_max.;
         v=1/k;
         if &i.=0 then
         pred_&i.=pred_zi + (exp(lgamma(&i.+(v))-lgamma(&i.+1)-lgamma (v) +    
             &i.*log((1/v)*pred) - (&i.+(v))*log(1+(1/v)*pred)))*(1-pred_zi);    
         else
            pred_&i.=(exp(lgamma(&i.+(v))-lgamma(&i.+1)-lgamma (v) +    
             &i.*log((1/v)*pred) - (&i.+(v))*log(1+(1/v)*pred)))*(1-pred_zi);
      %end;
   %mend;

   data pred3;
      merge pred_zidata preddata pred_k;
      array preds (*) pred_0-pred_&y_max.;
      %estimate;
      pred_sum = sum(of preds(*));
   run;
   %graphit;
%mend;
