*SAS Program 5.6.4-5;
OPTIONS NOCENTER PS=500 ls=150;
LIBNAME Lib 'd:\MLM\data';

Data data1;
 Set Lib.ML_data_Chap5;

PROC SQL;
Create table temp as select site, heavy_inj, 
 100*(mean(heavy_inj)) as m_heavy_inj, count(site) as Nj from data1
Group by site;
Create table data2 as select distinct site, m_heavy_inj, Nj,
m_heavy_inj>15 as Hinj_Region from temp;
Quit;

Data data3;
Merge data1 data2; by site;

*(1) Poisson;
Proc GLIMMIX data=data3 noitprint noclprint; 
Class site; 
ID site jailyear; 
Model jailyear=Hinj_Region|gender ethnic gdmc_age highsch
  /S dist=POISSON link=LOG DDFM=BW;
RANDOM int gender/subject=site g; 
NLOPTIONS tech=nrridg;
OUTPUT out=psstats pred(ilink)=p;
Run;
Data psstats(keep=site rp dist);
 Set psstats;
rp=round(p);
dist='Poisson';
Run;
Title 'Poisson Predictions';
Proc Print data=psstats; 
Run;

*(2) Overdispersed Possion;
Proc GLIMMIX data=data3 noitprint noclprint; 
Class site; 
Model jailyear=Hinj_Region|gender ethnic gdmc_age highsch
  /S dist=POISSON link=LOG DDFM=BW;
RANDOM Int gender/subject=site; 
RANDOM _residual_ /subject=site;
NLOPTIONS tech=nrridg;
OUTPUT out=psstats2 pred(ilink)=p;
Run;
Data psstats2(keep=site rp dist);
 Set psstats2;
rp=round(p);
dist='Poissonr';
Run;
title 'Overdispersed Poisson Predictions';
Proc print data=psstats2; 
Run;


*(3) Negative Binomial;
Proc GLIMMIX data=data3 noitprint noclprint; 
Class site; 
Model jailyear=Hinj_Region|gender ethnic gdmc_age highsch
  /S dist=NB link=LOG Ddfm=Bw;
RANDOM Int gender/subject=site g; 
NLOPTIONS tech=nrridg;
OUTPUT out=nbstats pred(ilink)=p;
Run;

Data nbstats(keep=rp dist);
 Set nbstats;
rp=round(p);
dist='NegBin';
Run;
Title 'Negative Binomial Predictions';
Proc Print data=nbstats;
Run;

*(4) Raw data;
Data rwstats(keep=rp dist);
 Set data3;
rename jailyear=rp;
dist='Observed';
Run;
Title 'Raw Data';
Proc Print data=rwstats;
Run;

*Combined;
Data combined;  
 Set rwstats psstats psstats2 nbstats;
Run;

Proc Freq data=combined; 
by dist notsorted;
Tables rp / out=graphstats;
Run;

Data graphstats;
 Set graphstats;
prob=percent/100;
Label prob='Probability' rp='Number of Events';
Run;

Proc Print data=graphstats;
Run;

symbol1 i=j v=star r=1 c=blue;
symbol2 i=j v=dot r=1 c=black;
symbol3 i=j v=square r=1 c=cyan;
symbol4 i=j v=circle r=1 c=red;
axis1 label=('Number of Events') order=(0 to 36 by 1);
axis2 label=(angle=90 h=1 f=swiss 'Probability');
Title 'Poisson, Overdispersed Poisson and Negative Binomial Model Fit Comparison';
Proc Gplot data=graphstats;
Plot prob*rp=dist / haxis=axis1 vaxis=axis2;
Run;
Quit;
